<?php

namespace Lc\CaracoleBundle\Model\Product;


use Doctrine\Common\Collections\Collection;
use Lc\CaracoleBundle\Model\Reduction\ReductionCatalogInterface;
use Lc\CaracoleBundle\Model\Section\SectionInterface;
use Lc\SovBundle\Model\File\FileInterface;

/**
 * @ORM\MappedSuperclass()
 */
interface ProductFamilyInterface
{
    public function getBehaviorCountStockChoices(): array;

    public function getBehaviorDisplaySaleChoices(): array;

    public function getBehaviorStockCycleChoices(): array;

    public function getWaringMessageTypeChoices(): array;

    public function getBehaviorAddToCartChoices(): array;

    public function getBehaviorPriceChoices(): array;

    public function getPropertyOrganicLabelChoices(): array;

    public function getTypeExpirationDateChoices(): array;

    public function getBehaviorExpirationDateChoices(): array;

    public function getSection(): SectionInterface;

    public function setSection(SectionInterface $section): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getAvailableQuantityInherited();

    public function getTaxRateInherited();

    public function getActiveProducts(): ?bool;

    public function setActiveProducts(bool $activeProducts): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getProductsQuantityAsTitle(): ?bool;

    public function setProductsQuantityAsTitle(bool $productsQuantityAsTitle
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getProductsType(): ?string;

    public function setProductsType(?string $productsType): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getQuantityLabel(): ?string;

    public function setQuantityLabel(?string $quantityLabel): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    /**
     * @return Collection|ProductInterface[]
     */
    public function getProducts(): Collection;

    public function getProductsOnline(): Collection;

    public function addProduct(ProductInterface $product): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function removeProduct(ProductInterface $product): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getReductionCatalog(): ?ReductionCatalogInterface;

    public function getReductionCatalogInherited(): ?ReductionCatalogInterface;

    public function setReductionCatalog(?ReductionCatalogInterface $reductionCatalog
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    /**
     * @return Collection|ProductCategoryInterface[]
     */
    public function getProductCategories(): Collection;

    public function initProductCategories();

    public function addProductCategory(ProductCategoryInterface $productCategory
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function removeProductCategory(ProductCategoryInterface $productCategory
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getProductCategoryParent();

    public function getProductCategoryChild();

    public function getSubtitle(): ?string;

    public function setSubtitle(?string $subtitle): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getWarningMessage(): ?string;

    public function setWarningMessage(?string $warningMessage): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getWarningMessageType(): ?string;

    public function setWarningMessageType(?string $warningMessageType
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getNote(): ?string;

    public function setNote(?string $note): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getBehaviorOutOfStock(): ?string;

    public function setBehaviorOutOfStock(?string $behaviorOutOfStock
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getBehaviorCountStock(): ?string;

    public function setBehaviorCountStock(string $behaviorCountStock
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getExportTitle(): ?string;

    public function setExportTitle(?string $exportTitle): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getExportNote(): ?string;

    public function setExportNote(?string $exportNote): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getBehaviorStockWeek(): ?string;

    public function setBehaviorStockWeek(string $behaviorStockWeek
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getAvailabilityRenewedThisWeek(): ?bool;

    public function setAvailabilityRenewedThisWeek(?bool $availabilityRenewedThisWeek
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getBehaviorDisplaySale(): ?string;

    public function setBehaviorDisplaySale(string $behaviorDisplaySale
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function isPropertyNoveltyOnline(): ?bool;

    public function getPropertyNoveltyExpirationDate(): ?\DateTimeInterface;

    public function setPropertyNoveltyExpirationDate(?\DateTimeInterface $propertyNoveltyExpirationDate
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getPropertyOrganicLabel(): ?string;

    public function setPropertyOrganicLabel(?string $propertyOrganicLabel
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getPropertyAllergens(): ?string;

    public function setPropertyAllergens(?string $propertyAllergens
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getPropertyComposition(): ?string;

    public function setPropertyComposition(?string $propertyComposition
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getPropertyFragrances(): ?string;

    public function setPropertyFragrances(?string $propertyFragrances
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function countProperties(): bool;

    public function getBehaviorExpirationDate(): ?string;

    public function setBehaviorExpirationDate(?string $behaviorExpirationDate
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getTypeExpirationDate(): ?string;

    public function setTypeExpirationDate(?string $typeExpirationDate
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getPropertyWeight(): ?string;

    public function setPropertyWeight(?string $propertyWeight): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getPropertyQuantity(): ?string;

    public function setPropertyQuantity(?string $propertyQuantity): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getPropertyVariety(): ?string;

    public function setPropertyVariety(?string $propertyVariety): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getPropertyFeature(): ?string;

    public function setPropertyFeature(?string $propertyFeature): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getPropertyAlcoholLevel(): ?string;

    public function setPropertyAlcoholLevel(?string $propertyAlcoholLevel
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getPropertyPackaging(): ?string;

    public function setPropertyPackaging(?string $propertyPackaging
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getPropertyCharacteristics(): ?string;

    public function setPropertyCharacteristics(?string $propertyCharacteristics
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getBehaviorAddToCart(): ?string;

    public function setBehaviorAddToCart(?string $behaviorAddToCart
    ): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getBehaviorPrice(): ?string;

    public function getBehaviorPriceInherited();

    public function setBehaviorPrice(?string $behaviorPrice): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function hasProductsWithVariousWeight();

    public function getProductsGroupByTitle();

    public function getOriginProduct();

    public function getOriginProductOnline();

    public function hasOneProductOnline();

    public function getSaleStatus(): ?bool;

    public function setSaleStatus(bool $saleStatus): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;

    public function getFieldBuyingPrice();

    public function getFieldPrice();

    public function getImage(): ?FileInterface;

    public function setImage(?FileInterface $image): \Lc\CaracoleBundle\Model\Product\ProductFamilyModel;
}