<?php

namespace Lc\CaracoleBundle\Repository\Product;

use Knp\Component\Pager\PaginatorInterface;
use Lc\CaracoleBundle\Model\Product\ProductCategoryInterface;
use Lc\SovBundle\Repository\AbstractRepositoryQuery;

class ProductFamilyRepositoryQuery extends AbstractRepositoryQuery implements RepositoryQuery
{
    protected bool $isJoinProductCategories;
    protected bool $isJoinProducts;

    public function __construct(ProductFamilyRepository $repository, PaginatorInterface $paginator)
    {
        parent::__construct($repository, 'r', $paginator);
    }

    public function joinProductCategories(): self
    {
        if (!$this->isJoinProductCategories) {
            $this->isJoinProductCategories = true;

            return $this
                    ->leftJoin('.productCategories', 'cat');
            //$query->addSelect('cat') ; ???
        }
        return $this;
    }


    public function joinProducts(): self
    {
        if (!$this->isJoinProducts) {
            $this->isJoinProducts = true;

            return $this
                    ->innerJoin('.products', 'pfp');
            //  $query->addSelect('pfp') ; ?????
        }
        return $this;
    }

    public function orderByPosition(): self
    {
        $this->orderBy('e.position', 'ASC');
        return $this;
    }

    public function filterByProductCategory(ProductCategoryInterface $category): self
    {
        $this->joinProductCategories();
        return $this
                ->andWhere(':category MEMBER OF .productCategories')
                ->setParameter('category', $category->getId());
    }


    public function filterByPropertyNoveltyExpirationDate($dateTime = null): self
    {
        if (is_null($dateTime)) {
            $dateTime = new \DateTime();
        }

        return $this->andWhere(':now <= e.propertyNoveltyExpirationDate')
                ->setParameter('now', $dateTime);
    }

    public function filterPropertyLargeVolume(): self
    {
        return $this->andWhere('.propertyLargeVolume = 1');
    }

    public function filterPropertyOrganicLabel(): self
    {
        return $this->andWhere('.propertyOrganicLabel IS NOT NULL');
    }

    public function filterByTerms($terms): self
    {
        $this->andWhere('.title LIKE :terms OR cat.title LIKE :terms');
        $this->setParameter(':terms', '%'.$terms.'%') ;
        return $this;
    }

}
